//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2005, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef INST_DATA_SECTION_HH
# define INST_DATA_SECTION_HH

# include <stdint.h>

# include <ostream>
# include <vector>
# include <string>

# include "misc/unique_string.hh"

# include "inst/section.hh"
# include "inst/fwd.hh"

namespace inst
{

  /// Abstract an assembly data section
  class DataSection:
    public Section
  {
  public:
    DataSection(const std::string &name = ".data");
    virtual ~DataSection();

  public:
    int size() const;

  public:
    bool define_label(const misc::unique_string &id);

  public:
    void add_space(unsigned space_size);

    void add_byte(int b);
    void add_word(int w);

  public:
    int load_byte(unsigned offset) const;

  public:
    virtual void write(std::ostream& ostr) const;

  protected:
    std::vector<const Label*> labels_;

    std::vector<uint8_t> bytes_;
  };

} // namespace inst

# include "inst/data_section.hxx"

#endif // !INST_DATA_SECTION_HH
