//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef INST_PROGRAM_BUILDER_HXX
# define INST_PROGRAM_BUILDER_HXX

# include "inst/program_builder.hh"

namespace inst
{

  //
  inline
  ProgramBuilder::ProgramBuilder():
    program_(0), nop_after_branch_p_(false)
  {
  }

  //
  inline void
  ProgramBuilder::reset()
  {
    program_ = new Program();
  }

  inline void
  ProgramBuilder::set_nop_after_branch(bool nop_after_branch_p)
  {
    nop_after_branch_p_ = nop_after_branch_p;
  }

  inline Program *
  ProgramBuilder::get()
  {
    // FIXME: fill end of program with NOPs for pipeline (really dirty!)
    for (unsigned i = 0; i < 6; ++i)
      program_->text_section().
        add_inst (new Sll(Register(Register::general, Cpu::zero),
                          Register(Register::general, Cpu::zero),
                          new IntExp(0)));
    return program_;
  }

  //
  inline void
  ProgramBuilder::define_inst_label(const misc::unique_string &id)
  {
    if (!program_->text_section ().define_label(id))
      {
        std::cerr << "Instruction label already defined in this section."
                  << std::endl;
        exit_set(exit_solve);
      }
  }

  inline void
  ProgramBuilder::define_data_label(const misc::unique_string &id)
  {
    if (!program_->data_section ().define_label(id))
      {
        std::cerr << "Data label already defined in this section."
                  << std::endl;
        exit_set(exit_solve);
      }
  }

  //
  inline void
  ProgramBuilder::add_byte(int imm)
  {
    program_->data_section ().add_byte(imm);
  }

  inline void
  ProgramBuilder::add_half(int imm)
  {
    // FIXME: check that imm is less than 0x10000!
    add_byte((imm >> 8) & 0xff);
    add_byte(imm & 0xff);
  }

  inline void
  ProgramBuilder::add_word(int imm)
  {
    program_->data_section ().add_word (imm);
  }
  inline void
  ProgramBuilder::add_word(LabelExp *l)
  {
    // FIXME: use a precondition
    program_->data_section ().
      add_word (program_->data_section ().get_offset(l->get_name()));
  }

  inline void
  ProgramBuilder::add_ascii(const std::string *str)
  {
    for (unsigned i = 0; i < str->size (); ++i)
      program_->data_section ().add_byte((*str)[i]);
  }
  inline void
  ProgramBuilder::add_asciiz(const std::string *str)
  {
    add_ascii(str);
    add_byte(0);
  }

  inline void
  ProgramBuilder::add_space (int size)
  {
    program_->data_section ().add_space(size);
  }

} // namespace inst

#endif // !INST_PROGRAM_BUILDER_HXX
