//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2004, 2005, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//

//
// WARNING: Generated file, do not edit by hand!
//

#include "inst/program_solver.hh"
#include "inst/int_exp.hh"
#include "inst/label_exp.hh"
#include "inst/op_exp.hh"

#include "vm/cpu.hh"

namespace inst
{

  //
  ProgramSolver::~ProgramSolver()
  {
  }

  //
  void
  ProgramSolver::visit(IntExp &)
  {
  }

  void
  ProgramSolver::visit(LabelExp &l)
  {
    if (! program_->data_section().has_label(l.get_name()))
      if (! program_->text_section().has_label(l.get_name()))
        {
          std::cerr << "Undefined label: " << l.get_name()
                    << std::endl;
          exit_set(exit_solve);
        }
      else
        l.set_immediate(program_->text_section().
                        get_offset(l.get_name()) - pc_);
    else
      l.set_immediate(program_->data_section().get_offset(l.get_name()));
  }

  void
  ProgramSolver::visit(OpExp &o)
  {
    o.get_left().accept(*this);
    o.get_right().accept(*this);
    switch (o.kind_)
      {
      case OpExp::add:
        o.set_immediate(o.get_left().get_immediate() +
                        o.get_right().get_immediate());
        break;
      case OpExp::sub:
        o.set_immediate(o.get_left().get_immediate() -
                        o.get_right().get_immediate());
        break;
      case OpExp::mul:
        o.set_immediate(o.get_left().get_immediate() *
                        o.get_right().get_immediate());
        break;
      case OpExp::div:
        o.set_immediate(o.get_left().get_immediate() /
                        o.get_right().get_immediate());
        break;
      }
  }

  void
  ProgramSolver::solve_cpu_register(Register &reg) const
  {
    switch (reg.get_kind())
      {
      case Register::generic:
        reg.set_kind(Register::general);
        /* fallthrough */
      case Register::general:
        if ((reg.get_index() == vm::Cpu::t9 && max_caller_save_ < 10)
            || (reg.get_index() == vm::Cpu::t8 && max_caller_save_ < 9)
            || (vm::Cpu::t0 + max_caller_save_ - 1 < reg.get_index()
                && reg.get_index() <= vm::Cpu::t7))
          {
            std::cerr << "Caller-save register limited." << std::endl;
            exit_set(exit_solve);
          }
        else if (vm::Cpu::s0 + max_callee_save_ - 1 < reg.get_index() &&
                 reg.get_index() <= vm::Cpu::s7)
          {
            std::cerr << "Callee-save register limited." << std::endl;
            exit_set(exit_solve);
          }
        else if (vm::Cpu::a0 + max_args_ - 1 < reg.get_index() &&
                 reg.get_index() <= vm::Cpu::a3)
          {
            std::cerr << "Argument register limited." << std::endl;
            exit_set(exit_solve);
          }
        break;
      case Register::unlimited:
        if (!unlimited_regs_enabled_)
          {
            std::cerr << "Unlimited registers prohibited." << std::endl;
            exit_set(exit_solve);
          }
        break;
      };
  }

  void
  ProgramSolver::solve_cop_register(Register &reg) const
  {
    if (reg.get_kind() != Register::generic)
      {
        std::cerr << "Coprocessor registers must be generic." << std::endl;
        exit_set(exit_solve);
      }
  }


  void
  ProgramSolver::visit(Add &add)
  {
    solve_cpu_register(add.get_dest());
    solve_cpu_register(add.get_src1());
    solve_cpu_register(add.get_src2());
  }
  void
  ProgramSolver::visit(Addu &addu)
  {
    solve_cpu_register(addu.get_dest());
    solve_cpu_register(addu.get_src1());
    solve_cpu_register(addu.get_src2());
  }
  void
  ProgramSolver::visit(Addi &addi)
  {
    solve_cpu_register(addi.get_dest());
    solve_cpu_register(addi.get_src());
    solve_exp(addi.get_imm());
  }
  void
  ProgramSolver::visit(Addiu &addiu)
  {
    solve_cpu_register(addiu.get_dest());
    solve_cpu_register(addiu.get_src());
    solve_exp(addiu.get_imm());
  }
  void
  ProgramSolver::visit(Sub &sub)
  {
    solve_cpu_register(sub.get_dest());
    solve_cpu_register(sub.get_src1());
    solve_cpu_register(sub.get_src2());
  }
  void
  ProgramSolver::visit(Subu &subu)
  {
    solve_cpu_register(subu.get_dest());
    solve_cpu_register(subu.get_src1());
    solve_cpu_register(subu.get_src2());
  }
  void
  ProgramSolver::visit(Mul &mul)
  {
    solve_cpu_register(mul.get_dest());
    solve_cpu_register(mul.get_src1());
    solve_cpu_register(mul.get_src2());
  }
  void
  ProgramSolver::visit(Div &div)
  {
    solve_cpu_register(div.get_src1());
    solve_cpu_register(div.get_src2());
  }
  void
  ProgramSolver::visit(Divu &divu)
  {
    solve_cpu_register(divu.get_src1());
    solve_cpu_register(divu.get_src2());
  }
  void
  ProgramSolver::visit(Sll &sll)
  {
    solve_cpu_register(sll.get_dest());
    solve_cpu_register(sll.get_src());
    solve_exp(sll.get_imm());
  }
  void
  ProgramSolver::visit(Sllv &sllv)
  {
    solve_cpu_register(sllv.get_dest());
    solve_cpu_register(sllv.get_src1());
    solve_cpu_register(sllv.get_src2());
  }
  void
  ProgramSolver::visit(Sra &sra)
  {
    solve_cpu_register(sra.get_dest());
    solve_cpu_register(sra.get_src());
    solve_exp(sra.get_imm());
  }
  void
  ProgramSolver::visit(Srav &srav)
  {
    solve_cpu_register(srav.get_dest());
    solve_cpu_register(srav.get_src1());
    solve_cpu_register(srav.get_src2());
  }
  void
  ProgramSolver::visit(Srl &srl)
  {
    solve_cpu_register(srl.get_dest());
    solve_cpu_register(srl.get_src());
    solve_exp(srl.get_imm());
  }
  void
  ProgramSolver::visit(Srlv &srlv)
  {
    solve_cpu_register(srlv.get_dest());
    solve_cpu_register(srlv.get_src1());
    solve_cpu_register(srlv.get_src2());
  }
  void
  ProgramSolver::visit(And &and_)
  {
    solve_cpu_register(and_.get_dest());
    solve_cpu_register(and_.get_src1());
    solve_cpu_register(and_.get_src2());
  }
  void
  ProgramSolver::visit(Andi &andi)
  {
    solve_cpu_register(andi.get_dest());
    solve_cpu_register(andi.get_src());
    solve_exp(andi.get_imm());
  }
  void
  ProgramSolver::visit(Or &or_)
  {
    solve_cpu_register(or_.get_dest());
    solve_cpu_register(or_.get_src1());
    solve_cpu_register(or_.get_src2());
  }
  void
  ProgramSolver::visit(Ori &ori)
  {
    solve_cpu_register(ori.get_dest());
    solve_cpu_register(ori.get_src());
    solve_exp(ori.get_imm());
  }
  void
  ProgramSolver::visit(Xor &xor_)
  {
    solve_cpu_register(xor_.get_dest());
    solve_cpu_register(xor_.get_src1());
    solve_cpu_register(xor_.get_src2());
  }
  void
  ProgramSolver::visit(Xori &xori)
  {
    solve_cpu_register(xori.get_dest());
    solve_cpu_register(xori.get_src());
    solve_exp(xori.get_imm());
  }
  void
  ProgramSolver::visit(Nor &nor)
  {
    solve_cpu_register(nor.get_dest());
    solve_cpu_register(nor.get_src1());
    solve_cpu_register(nor.get_src2());
  }
  void
  ProgramSolver::visit(Slt &slt)
  {
    solve_cpu_register(slt.get_dest());
    solve_cpu_register(slt.get_src1());
    solve_cpu_register(slt.get_src2());
  }
  void
  ProgramSolver::visit(Sltu &sltu)
  {
    solve_cpu_register(sltu.get_dest());
    solve_cpu_register(sltu.get_src1());
    solve_cpu_register(sltu.get_src2());
  }
  void
  ProgramSolver::visit(Slti &slti)
  {
    solve_cpu_register(slti.get_dest());
    solve_cpu_register(slti.get_src());
    solve_exp(slti.get_imm());
  }
  void
  ProgramSolver::visit(Sltiu &sltiu)
  {
    solve_cpu_register(sltiu.get_dest());
    solve_cpu_register(sltiu.get_src());
    solve_exp(sltiu.get_imm());
  }
  void
  ProgramSolver::visit(Beq &beq)
  {
    solve_cpu_register(beq.get_src1());
    solve_cpu_register(beq.get_src2());
    solve_exp(beq.get_label());
  }
  void
  ProgramSolver::visit(Bne &bne)
  {
    solve_cpu_register(bne.get_src1());
    solve_cpu_register(bne.get_src2());
    solve_exp(bne.get_label());
  }
  void
  ProgramSolver::visit(Bgez &bgez)
  {
    solve_cpu_register(bgez.get_src());
    solve_exp(bgez.get_label());
  }
  void
  ProgramSolver::visit(Bgezal &bgezal)
  {
    solve_cpu_register(bgezal.get_src());
    solve_exp(bgezal.get_label());
  }
  void
  ProgramSolver::visit(Bgtz &bgtz)
  {
    solve_cpu_register(bgtz.get_src());
    solve_exp(bgtz.get_label());
  }
  void
  ProgramSolver::visit(Blez &blez)
  {
    solve_cpu_register(blez.get_src());
    solve_exp(blez.get_label());
  }
  void
  ProgramSolver::visit(Bltz &bltz)
  {
    solve_cpu_register(bltz.get_src());
    solve_exp(bltz.get_label());
  }
  void
  ProgramSolver::visit(Bltzal &bltzal)
  {
    solve_cpu_register(bltzal.get_src());
    solve_exp(bltzal.get_label());
  }
  void
  ProgramSolver::visit(Jmp &j)
  {
    solve_exp(j.get_label());
  }
  void
  ProgramSolver::visit(Jal &jal)
  {
    solve_exp(jal.get_label());
  }
  void
  ProgramSolver::visit(Jr &jr)
  {
    solve_cpu_register(jr.get_dest());
  }
  void
  ProgramSolver::visit(Jalr &jalr)
  {
    solve_cpu_register(jalr.get_dest());
  }
  void
  ProgramSolver::visit(Lb &lb)
  {
    solve_cpu_register(lb.get_dest());
    solve_exp(lb.get_offset());
    solve_cpu_register(lb.get_base());
  }
  void
  ProgramSolver::visit(Lbu &lbu)
  {
    solve_cpu_register(lbu.get_dest());
    solve_exp(lbu.get_offset());
    solve_cpu_register(lbu.get_base());
  }
  void
  ProgramSolver::visit(Lw &lw)
  {
    solve_cpu_register(lw.get_dest());
    solve_exp(lw.get_offset());
    solve_cpu_register(lw.get_base());
  }
  void
  ProgramSolver::visit(Lui &lui)
  {
    solve_cpu_register(lui.get_dest());
    solve_exp(lui.get_imm());
  }
  void
  ProgramSolver::visit(Sb &sb)
  {
    solve_cpu_register(sb.get_src());
    solve_exp(sb.get_offset());
    solve_cpu_register(sb.get_base());
  }
  void
  ProgramSolver::visit(Sw &sw)
  {
    solve_cpu_register(sw.get_src());
    solve_exp(sw.get_offset());
    solve_cpu_register(sw.get_base());
  }
  void
  ProgramSolver::visit(Mfhi &mfhi)
  {
    solve_cpu_register(mfhi.get_dest());
  }
  void
  ProgramSolver::visit(Mflo &mflo)
  {
    solve_cpu_register(mflo.get_dest());
  }
  void
  ProgramSolver::visit(Mthi &mthi)
  {
    solve_cpu_register(mthi.get_src());
  }
  void
  ProgramSolver::visit(Mtlo &mtlo)
  {
    solve_cpu_register(mtlo.get_src());
  }
  void
  ProgramSolver::visit(Mfc0 &mfc0)
  {
    solve_cpu_register(mfc0.get_dest());
    solve_cop_register(mfc0.get_src());
  }
  void
  ProgramSolver::visit(Mtc0 &mtc0)
  {
    solve_cop_register(mtc0.get_dest());
    solve_cpu_register(mtc0.get_src());
  }
  void
  ProgramSolver::visit(Syscall &)
  {
  }

} // namespace inst
