//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
/** \file vm/memory.hh
    \brief Declare Memory class */
#ifndef VM_MEMORY_HH
# define VM_MEMORY_HH

# include <stdint.h>

# include "inst/fwd.hh"

# include "vm/segment.hh"

namespace vm
{

  /// Memory abstraction
  class Memory
  {
  public:
    static const int stack_bottom = INT32_MAX - 3;
    static const int default_stack_size = 512 * 1024;

    enum segment_kind
      {
        heap = 0,
        stack = 1
      };

    /** \name Constructor and destructor
        \{ */
  public:
    /// Construct a Memory
    Memory(int stack_size = default_stack_size);

    /// Reset the memory
    void reset();

  public:
    int sbrk(int size);

  public:
    int heap_size() const;
    int stack_size() const;

    int get_stack_top() const;

    /** \name Store instructions
        \{ */
  public:
    void store(const inst::DataSection& data_section);

    /// Store a byte in memory
    void store_byte(segment_kind k, int offset, int b);
    /// Store a word in memory
    void store_word(segment_kind k, int offset, int w);
    /** \} */

    /** \name Load instructions
        \{ */
  public:
    /// Load a byte from memory
    int load_byte(segment_kind k, int offset) const;
    /// Load a word from memory
    int load_word(segment_kind k, int offset) const;
    /** \} */

  protected:
    Segment segments_[2];

    const int stack_top_;
  };

} // namespace vm

# include "vm/memory.hxx"

#endif // !VM_MEMORY_HH
