//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef VM_SEGMENT_HXX
# define VM_SEGMENT_HXX

# include "vm/segment.hh"

# include "misc/contract.hh"

namespace vm
{

  inline
  Segment::Segment(int size):
    size_(size), bytes_(0)
  {
    if (size)
      resize(size);
  }
  inline
  Segment::~Segment()
  {
    if (bytes_)
      delete [] bytes_;
  }

  //
  inline void
  Segment::resize(int new_size)
  {
    uint8_t*            new_bytes = new uint8_t[new_size];
    if (bytes_)
      {
        int             min = new_size < size_ ? new_size : size_;
        // FIXME: may be accelerated with memcpy
        for (int i = 0; i < min; ++i)
          new_bytes[i] = bytes_[i];
        delete [] bytes_;
      }
    bytes_ = new_bytes;
    size_ = new_size;
  }

  inline int
  Segment::size() const
  {
    return size_;
  }


  //
  inline void
  Segment::store_byte(int offset, int b)
  {
    assertion(offset < size_);
    bytes_[offset] = (uint8_t)(b);
  }

  inline void
  Segment::store_word(int offset, int w)
  {
    unsigned            pow = 256 * 256 * 256;
    for (unsigned i = 0; i < 4; ++i, ++offset, pow /= 256)
      {
        assertion(offset < size_);
        bytes_[offset] = w / pow;
      }
  }

  //
  inline int
  Segment::load_byte(int offset) const
  {
    assertion(offset < size_);
    return (int8_t)bytes_[offset];
  }

  inline int
  Segment::load_word(int offset) const
  {
    int         w = 0;
    for (unsigned i = 0; i < 4; ++i, ++offset)
      {
        assertion(offset < size_);
        w = (w * 256) + bytes_[offset];
      }
    return w;
  }

} // namespace vm

#endif // !VM_SEGMENT_HXX
